@@ -16,23 +16,61 @@
 //! This framework is primarily used with HTTP, as suggested by the name. It is
 //! also used by some other protocols such as RTSP.
 //!
-//! Quick example:
+//! ## Cargo Features
 //!
-//! ```rust
-//! use std::convert::TryFrom;
-//! let WWW_AUTHENTICATE = "UnsupportedSchemeA, Basic realm=\"foo\", UnsupportedSchemeB";
-//! let mut pw_client = http_auth::PasswordClient::try_from(WWW_AUTHENTICATE).unwrap();
-//! assert!(matches!(pw_client, http_auth::PasswordClient::Basic(_)));
-//! let response = pw_client.respond(&http_auth::PasswordParams {
-//!     username: "Aladdin",
-//!     password: "open sesame",
-//!     uri: "/",
-//!     method: "GET",
-//!     body: Some(&[]),
-//! }).unwrap();
-//! assert_eq!(response, "Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==");
-//! ```
+//! | feature         | default? | description                                     |
+//! |-----------------|----------|-------------------------------------------------|
+//! | `basic-scheme`  | yes      | support for the `Basic` auth scheme             |
+//! | `digest-scheme` | yes      | support for the `Digest` auth scheme            |
+//! | `http`          | no       | convenient conversion from [`http`] crate types |
+//!
+//! ## Example
+//!
+//! In most cases, callers only need to use [`PasswordClient`] and
+//! [`PasswordParams`] to handle `Basic` and `Digest` authentication schemes.
+//!
+#![cfg_attr(
+    feature = "http",
+    doc = r##"
+```rust
+use std::convert::TryFrom as _;
+use http_auth::PasswordClient;
 
+let WWW_AUTHENTICATE_VAL = "UnsupportedSchemeA, Basic realm=\"foo\", UnsupportedSchemeB";
+let mut pw_client = http_auth::PasswordClient::try_from(WWW_AUTHENTICATE_VAL).unwrap();
+assert!(matches!(pw_client, http_auth::PasswordClient::Basic(_)));
+let response = pw_client.respond(&http_auth::PasswordParams {
+    username: "Aladdin",
+    password: "open sesame",
+    uri: "/",
+    method: "GET",
+    body: Some(&[]),
+}).unwrap();
+assert_eq!(response, "Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==");
+```
+"##
+)]
+//!
+//! The `http` feature allows parsing all `WWW-Authenticate` headers within a
+//! [`http::HeaderMap`] in one call.
+//!
+#![cfg_attr(
+    feature = "http",
+    doc = r##"
+```rust
+# use std::convert::TryFrom as _;
+use http::header::{HeaderMap, WWW_AUTHENTICATE};
+# use http_auth::PasswordClient;
+
+let mut headers = HeaderMap::new();
+headers.append(WWW_AUTHENTICATE, "UnsupportedSchemeA".parse().unwrap());
+headers.append(WWW_AUTHENTICATE, "Basic realm=\"foo\", UnsupportedSchemeB".parse().unwrap());
+
+let mut pw_client = PasswordClient::try_from(headers.get_all(WWW_AUTHENTICATE)).unwrap();
+assert!(matches!(pw_client, http_auth::PasswordClient::Basic(_)));
+```
+"##
+)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
 use std::convert::TryFrom;
@@ -78,9 +116,12 @@ fn char_classes(b: u8) -> u8 {
 }
 
 /// Parsed challenge (scheme and body) using references to the original header value.
+/// Produced by [`crate::parser::ChallengeParser`].
 ///
 /// This is not directly useful for responding to a challenge; it's an
-/// intermediary for constructing a [`PasswordClient`] or the like.
+/// intermediary for constructing a client that knows how to respond to a specific
+/// challenge scheme. In most cases, callers should construct a [`PasswordClient`]
+/// without directly using `ChallengeRef`.
 ///
 /// Only supports the param form, not the apocryphal `token68` form, as described
 /// in [`crate::parser::ChallengeParser`].
@@ -92,11 +133,10 @@ pub struct ChallengeRef<'i> {
     /// Zero or more parameters.
     ///
     /// These are represented as a `Vec` of key-value pairs rather than a
-    /// `HashMap`. Given that the parameters are generally only used once when
+    /// map. Given that the parameters are generally only used once when
     /// constructing a challenge client and each challenge only supports a few
     /// parameter types, it's more efficient in terms of CPU usage and code size
-    /// to scan through them directly without constructing a throw-away
-    /// `HashMap`.
+    /// to scan through them directly.
     pub params: Vec<ChallengeParamRef<'i>>,
 }
 
@@ -130,7 +170,18 @@ impl<'i> std::fmt::Debug for ParamsPrinter<'i> {
     }
 }
 
-/// Builds a [`PasswordClient`] from the supplied challenges.
+/// Builds a [`PasswordClient`] from the supplied challenges; create via
+/// [`PasswordClient::builder`].
+///
+/// Often you can just use [`PasswordClient`]'s [`TryFrom`] implementations
+/// to convert from a parsed challenge ([`crate::ChallengeRef`]) or
+/// unparsed challenges (`str`, [`http::header::HeaderValue`], or
+/// [`http::header::GetAll`]).
+///
+/// The builder allows more flexibility. For example, if you are using a HTTP
+/// library which is not based on a `http` crate, you might need to create
+/// a `PasswordClient` from an iterator over multiple `WWW-Authenticate`
+/// headers. You can feed each to [`PasswordClientBuilder::challenges`].
 ///
 /// Prefers `Digest` over `Basic`, consistent with the [RFC 7235 section
 /// 2.1](https://datatracker.ietf.org/doc/html/rfc7235#section-2.1) advice
@@ -141,11 +192,39 @@ impl<'i> std::fmt::Debug for ParamsPrinter<'i> {
 /// 3.7](https://datatracker.ietf.org/doc/html/rfc7616#section-3.7)
 /// advice to "use the first challenge it supports, unless a local policy
 /// dictates otherwise". In the future, it may prioritize by algorithm.
+///
+/// Ignores parse errors as long as there's at least one parseable, supported
+/// challenge.
+///
+/// ## Example
+///
+#[cfg_attr(
+    feature = "digest",
+    doc = r##"
+```rust
+use http_auth::PasswordClient;
+let client = PasswordClient::builder()
+    .challenges("UnsupportedSchemeA, Basic realm=\"foo\", UnsupportedSchemeB")
+    .challenges("Digest \
+                 realm=\"http-auth@example.org\", \
+                 qop=\"auth, auth-int\", \
+                 algorithm=MD5, \
+                 nonce=\"7ypf/xlj9XXwfDPEoM4URrv/xwf94BcCAzFZH4GiTo0v\", \
+                 opaque=\"FQhe/qaU925kfnzjCev0ciny7QMkPqMAFRtzCUYo5tdS\"")
+    .build()
+    .unwrap();
+assert!(matches!(client, PasswordClient::Digest(_)));
+```
+"##
+)]
 #[derive(Default)]
-pub struct PasswordClientBuilder {
-    first_err: Option<String>,
-    cur_client: Option<PasswordClient>,
-}
+pub struct PasswordClientBuilder(
+    /// The current result:
+    /// *   `Some(Ok(_))` if there is a suitable client.
+    /// *   `Some(Err(_))` if there is no suitable client and has been a parse error.
+    /// *   `None` otherwise.
+    Option<Result<PasswordClient, String>>,
+);
 
 impl PasswordClientBuilder {
     /// Considers all challenges from the given [`http::HeaderValue`] challenge list.
@@ -158,11 +237,8 @@ impl PasswordClientBuilder {
 
         match value.to_str() {
             Ok(v) => self = self.challenges(v),
-            Err(_) => {
-                if self.first_err.is_none() {
-                    self.first_err = Some("non-ASCII header value".into());
-                }
-            }
+            Err(_) if matches!(self.0, None) => self.0 = Some(Err("non-ASCII header value".into())),
+            _ => {}
         }
 
         self
@@ -171,13 +247,13 @@ impl PasswordClientBuilder {
     /// Returns true if no more challenges need to be examined.
     #[cfg(feature = "digest-scheme")]
     fn complete(&self) -> bool {
-        matches!(self.cur_client, Some(PasswordClient::Digest(_)))
+        matches!(self.0, Some(Ok(PasswordClient::Digest(_))))
     }
 
     /// Returns true if no more challenges need to be examined.
     #[cfg(not(feature = "digest-scheme"))]
     fn complete(&self) -> bool {
-        matches!(self.cur_client, Some(_))
+        matches!(self.0, Some(Ok(_)))
     }
 
     /// Considers all challenges from the given `&str` challenge list.
@@ -186,12 +262,8 @@ impl PasswordClientBuilder {
         while !self.complete() {
             match parser.next() {
                 Some(Ok(c)) => self = self.challenge(&c),
-                Some(Err(e)) => {
-                    if self.first_err.is_none() {
-                        self.first_err = Some(e.to_string());
-                    }
-                }
-                None => break,
+                Some(Err(e)) if self.0.is_none() => self.0 = Some(Err(e.to_string())),
+                _ => break,
             }
         }
         self
@@ -206,27 +278,25 @@ impl PasswordClientBuilder {
         #[cfg(feature = "digest-scheme")]
         if challenge.scheme.eq_ignore_ascii_case("Digest") {
             match DigestClient::try_from(challenge) {
-                Ok(c) => self.cur_client = Some(PasswordClient::Digest(c)),
-                Err(e) => {
-                    self.first_err.get_or_insert(e);
-                }
+                Ok(c) => self.0 = Some(Ok(PasswordClient::Digest(c))),
+                Err(e) if self.0.is_none() => self.0 = Some(Err(e)),
+                _ => {}
             }
             return self;
         }
 
         #[cfg(feature = "basic-scheme")]
-        if challenge.scheme.eq_ignore_ascii_case("Basic") && self.cur_client.is_none() {
+        if challenge.scheme.eq_ignore_ascii_case("Basic") && !matches!(self.0, Some(Ok(_))) {
             match BasicClient::try_from(challenge) {
-                Ok(c) => self.cur_client = Some(PasswordClient::Basic(c)),
-                Err(e) => {
-                    self.first_err.get_or_insert(e);
-                }
+                Ok(c) => self.0 = Some(Ok(PasswordClient::Basic(c))),
+                Err(e) if self.0.is_none() => self.0 = Some(Err(e)),
+                _ => {}
             }
             return self;
         }
 
-        if self.first_err.is_none() {
-            self.first_err = Some(format!("Unsupported scheme {:?}", challenge.scheme));
+        if self.0.is_none() {
+            self.0 = Some(Err(format!("Unsupported scheme {:?}", challenge.scheme)));
         }
 
         self
@@ -234,17 +304,18 @@ impl PasswordClientBuilder {
 
     /// Returns a new [`PasswordClient`] or fails.
     pub fn build(self) -> Result<PasswordClient, String> {
-        if let Some(c) = self.cur_client {
-            return Ok(c);
-        }
-        if let Some(e) = self.first_err {
-            return Err(e);
-        }
-        Err("no challenges given".into())
+        self.0.unwrap_or_else(|| Err("no challenges given".into()))
     }
 }
 
 /// Client for responding to a password challenge.
+///
+/// Typically created via [`TryFrom`] implementations for a parsed challenge
+/// ([`crate::ChallengeRef`]) or unparsed challenges (`str`,
+/// [`http::header::HeaderValue`], or [`http::header::GetAll`]). See full
+/// example in the [crate-level documentation](crate).
+///
+/// For more complex scenarios, see [`PasswordClientBuilder`].
 #[derive(Debug, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum PasswordClient {
@@ -322,6 +393,8 @@ impl TryFrom<http::header::GetAll<'_, http::HeaderValue>> for PasswordClient {
 
 impl PasswordClient {
     /// Builds a new `PasswordClient`.
+    ///
+    /// See example at [`PasswordClientBuilder`].
     pub fn builder() -> PasswordClientBuilder {
         PasswordClientBuilder::default()
     }
@@ -405,63 +478,128 @@ pub struct PasswordParams<'a> {
 
 /// Parses a list of challenges into a `Vec`.
 ///
-/// This is a shorthand for `parser::ChallengeParser::new(input).collect()`.
+/// Most callers don't need to directly parse; see [`PasswordClient`] instead.
+///
+/// This is a shorthand for `parser::ChallengeParser::new(input).collect()`. Use
+/// [`crate::parser::ChallengeParser`] directly when you want to parse lazily,
+/// avoid allocation, and/or see any well-formed challenges before an error.
+///
+/// ## Example
+///
+/// ```rust
+/// use http_auth::{parse_challenges, ChallengeRef, ParamValue};
+///
+/// // When all challenges are well-formed, returns them.
+/// assert_eq!(
+///     parse_challenges("UnsupportedSchemeA, Basic realm=\"foo\"").unwrap(),
+///     vec![
+///         ChallengeRef {
+///             scheme: "UnsupportedSchemeA",
+///             params: vec![],
+///         },
+///         ChallengeRef {
+///             scheme: "Basic",
+///             params: vec![("realm", ParamValue::try_from_escaped("foo").unwrap())],
+///         },
+///     ],
+/// );
+///
+/// // Returns `Err` if there is a syntax error anywhere in the input.
+/// parse_challenges("UnsupportedSchemeA, Basic realm=\"foo\", error error").unwrap_err();
+/// ```
 #[inline]
 pub fn parse_challenges(input: &str) -> Result<Vec<ChallengeRef>, parser::Error> {
     parser::ChallengeParser::new(input).collect()
 }
 
-/// Parsed parameter value.
+/// Parsed challenge parameter value used within [`ChallengeRef`].
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub struct ParamValue<'i> {
     /// The number of backslash escapes in a quoted-text parameter; 0 for a plain token.
     escapes: usize,
 
-    /// The raw string, which must be pure ASCII (no bytes >= 128) and be consistent with `escapes`.
-    raw: &'i str,
+    /// The escaped string, which must be pure ASCII (no bytes >= 128) and be
+    /// consistent with `escapes`.
+    escaped: &'i str,
 }
 
 impl<'i> ParamValue<'i> {
+    /// Tries to create a new `ParamValue` from an escaped sequence, primarily for testing.
+    ///
+    /// Validates the sequence and counts the number of escapes.
+    pub fn try_from_escaped(escaped: &'i str) -> Result<Self, String> {
+        let mut escapes = 0;
+        let mut pos = 0;
+        while pos < escaped.len() {
+            let slash = memchr::memchr(b'\\', &escaped.as_bytes()[pos..]).map(|off| pos + off);
+            for i in pos..slash.unwrap_or(escaped.len()) {
+                if (char_classes(escaped.as_bytes()[i]) & C_QDTEXT) == 0 {
+                    return Err(format!("{:?} has non-qdtext at byte {}", escaped, i));
+                }
+            }
+            if let Some(slash) = slash {
+                escapes += 1;
+                if escaped.len() <= slash + 1 {
+                    return Err(format!("{:?} ends at a quoted-pair escape", escaped));
+                }
+                if (char_classes(escaped.as_bytes()[slash + 1]) & C_ESCAPABLE) == 0 {
+                    return Err(format!(
+                        "{:?} has an invalid quote-pair escape at byte {}",
+                        escaped,
+                        slash + 1
+                    ));
+                }
+                pos = slash + 2;
+            } else {
+                break;
+            }
+        }
+        Ok(Self { escaped, escapes })
+    }
+
     /// Creates a new param, panicking if invariants are not satisfied.
     /// This not part of the stable API; it's just for the fuzz tester to use.
     #[doc(hidden)]
-    pub fn new(escapes: usize, raw: &'i str) -> Self {
+    pub fn new(escapes: usize, escaped: &'i str) -> Self {
         let mut pos = 0;
         for escape in 0..escapes {
-            match memchr::memchr(b'\\', &raw.as_bytes()[pos..]) {
+            match memchr::memchr(b'\\', &escaped.as_bytes()[pos..]) {
                 Some(rel_pos) => pos += rel_pos + 2,
                 None => panic!(
                     "expected {} backslashes in {:?}, ran out after {}",
-                    escapes, raw, escape
+                    escapes, escaped, escape
                 ),
             };
         }
-        if memchr::memchr(b'\\', &raw.as_bytes()[pos..]).is_some() {
-            panic!("expected {} backslashes in {:?}, are more", escapes, raw);
+        if memchr::memchr(b'\\', &escaped.as_bytes()[pos..]).is_some() {
+            panic!(
+                "expected {} backslashes in {:?}, are more",
+                escapes, escaped
+            );
         }
-        ParamValue { escapes, raw }
+        ParamValue { escapes, escaped }
     }
 
     /// Appends the unescaped form of this parameter to the supplied string.
-    fn append_unescaped(&self, to: &mut String) {
-        to.reserve(self.raw.len() - self.escapes);
+    pub fn append_unescaped(&self, to: &mut String) {
+        to.reserve(self.escaped.len() - self.escapes);
         let mut first_unwritten = 0;
         for _ in 0..self.escapes {
-            let i = match memchr::memchr(b'\\', &self.raw.as_bytes()[first_unwritten..]) {
+            let i = match memchr::memchr(b'\\', &self.escaped.as_bytes()[first_unwritten..]) {
                 Some(rel_i) => first_unwritten + rel_i,
                 None => panic!("bad ParamValues; not as many backslash escapes as promised"),
             };
-            to.push_str(&self.raw[first_unwritten..i]);
-            to.push_str(&self.raw[i + 1..i + 2]);
+            to.push_str(&self.escaped[first_unwritten..i]);
+            to.push_str(&self.escaped[i + 1..i + 2]);
             first_unwritten = i + 2;
         }
-        to.push_str(&self.raw[first_unwritten..]);
+        to.push_str(&self.escaped[first_unwritten..]);
     }
 
     /// Returns the unescaped length of this parameter; cheap.
     #[inline]
     pub fn unescaped_len(&self) -> usize {
-        self.raw.len() - self.escapes
+        self.escaped.len() - self.escapes
     }
 
     /// Returns the unescaped form of this parameter as a fresh `String`.
@@ -470,11 +608,32 @@ impl<'i> ParamValue<'i> {
         self.append_unescaped(&mut to);
         to
     }
+
+    /// Returns the unescaped form of this parameter, possibly appending it to `scratch`.
+    #[cfg(feature = "digest-scheme")]
+    fn unescaped_with_scratch<'tmp>(&self, scratch: &'tmp mut String) -> &'tmp str
+    where
+        'i: 'tmp,
+    {
+        if self.escapes == 0 {
+            self.escaped
+        } else {
+            let start = scratch.len();
+            self.append_unescaped(scratch);
+            &scratch[start..]
+        }
+    }
+
+    /// Returns the escaped string, unquoted.
+    #[inline]
+    pub fn as_escaped(&self) -> &'i str {
+        self.escaped
+    }
 }
 
 impl<'i> std::fmt::Debug for ParamValue<'i> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "\"{}\"", self.raw)
+        write!(f, "\"{}\"", self.escaped)
     }
 }
 
@@ -517,12 +676,33 @@ mod tests {
         }
     }
 
+    #[test]
+    fn try_from_escaped() {
+        assert_eq!(ParamValue::try_from_escaped("").unwrap().escapes, 0);
+        assert_eq!(ParamValue::try_from_escaped("foo").unwrap().escapes, 0);
+        assert_eq!(ParamValue::try_from_escaped("\\\"").unwrap().escapes, 1);
+        assert_eq!(
+            ParamValue::try_from_escaped("foo\\\"bar").unwrap().escapes,
+            1
+        );
+        assert_eq!(
+            ParamValue::try_from_escaped("foo\\\"bar\\\"baz")
+                .unwrap()
+                .escapes,
+            2
+        );
+        ParamValue::try_from_escaped("\\").unwrap_err(); // ends in slash
+        ParamValue::try_from_escaped("\"").unwrap_err(); // not valid qdtext
+        ParamValue::try_from_escaped("\n").unwrap_err(); // not valid qdtext
+        ParamValue::try_from_escaped("\\\n").unwrap_err(); // not valid escape
+    }
+
     #[test]
     fn unescape() {
         assert_eq!(
             &ParamValue {
                 escapes: 0,
-                raw: ""
+                escaped: ""
             }
             .to_unescaped(),
             ""
@@ -530,7 +710,7 @@ mod tests {
         assert_eq!(
             &ParamValue {
                 escapes: 0,
-                raw: "foo"
+                escaped: "foo"
             }
             .to_unescaped(),
             "foo"
@@ -538,7 +718,7 @@ mod tests {
         assert_eq!(
             &ParamValue {
                 escapes: 1,
-                raw: "\\foo"
+                escaped: "\\foo"
             }
             .to_unescaped(),
             "foo"
@@ -546,7 +726,7 @@ mod tests {
         assert_eq!(
             &ParamValue {
                 escapes: 1,
-                raw: "fo\\o"
+                escaped: "fo\\o"
             }
             .to_unescaped(),
             "foo"
@@ -554,7 +734,7 @@ mod tests {
         assert_eq!(
             &ParamValue {
                 escapes: 1,
-                raw: "foo\\bar"
+                escaped: "foo\\bar"
             }
             .to_unescaped(),
             "foobar"
@@ -562,7 +742,7 @@ mod tests {
         assert_eq!(
             &ParamValue {
                 escapes: 3,
-                raw: "\\foo\\ba\\r"
+                escaped: "\\foo\\ba\\r"
             }
             .to_unescaped(),
             "foobar"
